/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.binding;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;

import org.objectweb.fractal.julia.Controller;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.control.binding.ContentBindingController;

/**
 * Provides a container based implementation of the
 * {@link ContextualBindingController}interface. This mixin implements the
 * {@link ContextualBindingController}methods through delegation to the
 * encapsulated "user component". <br>
 * <b>Note: </b> this mixin cannot be used when the content and controllers of
 * components are merged. <br>
 * <b>Requirements </b>
 * <ul>
 * <li>none.</li>
 * </ul>
 */

public abstract class ContainerContextualBindingControllerMixin
    implements
      Controller,
      ContextualBindingController
{
  ThreadLocal<Object> hints;

  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private ContainerContextualBindingControllerMixin()
  {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overridden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * The "user component" encapsulated in this container component.
   */

  public Object fcContent;

  /**
   * Initializes the fields of this mixin and then calls the overridden method.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */

  public void initFcController(final InitializationContext ic)
      throws InstantiationException
  {
    hints = new ThreadLocal<Object>();
    fcContent = ic.content;
    Component owner = (Component) ic.getOptionalInterface("component");
    if (owner != null)
    {
      try
      {
        owner = (Component) owner.getFcInterface("component");
        if (fcContent == this)
        {
          // case of merge...AndContent options
          if (this instanceof ContentBindingController)
          {
            ((ContentBindingController) this).bindFcContent("component", owner);
          }
        }
        else if (fcContent instanceof BindingController)
        {
          ((BindingController) fcContent).bindFc("component", owner);
        }
      }
      catch (Exception ignored)
      {
      }
    }
    _super_initFcController(ic);
  }

  /**
   * @see BindingController#listFc()
   */
  public String[] listFc()
  {
    if (fcContent == this)
    {
      // case of merge...AndContent options
      if (this instanceof ContentBindingController)
      {
        return ((ContentBindingController) this).listFcContent();
      }
    }
    else if (fcContent instanceof BindingController)
    {
      return ((BindingController) fcContent).listFc();
    }
    return new String[0];
  }

  /**
   * @see BindingController#lookupFc(String)
   */
  public Object lookupFc(final String clientItfName)
      throws NoSuchInterfaceException
  {
    if (fcContent == this)
    {
      // case of merge...AndContent options
      ContentBindingController cbc = (ContentBindingController) this;
      return cbc.lookupFcContent(clientItfName);
    }
    BindingController bc = (BindingController) fcContent;
    return bc.lookupFc(clientItfName);
  }

  /**
   * @see BindingController#bindFc(String, Object)
   */
  public void bindFc(final String clientItfName, final Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (fcContent == this)
    {
      // case of merge...AndContent options
      ((ContentBindingController) this).bindFcContent(clientItfName, serverItf);
    }
    else
    {
      Object hints = this.hints.get();
      if (hints == null)
      {
        ((BindingController) fcContent).bindFc(clientItfName, serverItf);
      }
      else
      {
        ((ContextualBindingController) fcContent).bindFc(clientItfName,
            serverItf, hints);
      }
    }
  }

  /**
   * @see ContextualBindingController#bindFc(String, Object, Object)
   */
  public void bindFc(String clientItfName, Object serverItf, Object hints)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    this.hints.set(hints);
    bindFc(clientItfName, serverItf);
    this.hints.set(null);
  }

  /**
   * @see BindingController#unbindFc(String)
   */
  public void unbindFc(final String clientItfName)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    if (fcContent == this)
    {
      // case of merge...AndContent options
      ((ContentBindingController) this).unbindFcContent(clientItfName);
    }
    else
    {
      ((BindingController) fcContent).unbindFc(clientItfName);
    }
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The {@link Controller#initFcController initFcController}method overridden
   * by this mixin.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */

  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;
}
