/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Utility methods for content based message comparison.
 */
public final class MessageUtil
{
  private MessageUtil()
  {
  }

  /**
   * Returns <code>true</code> if the two messages has the same content. This
   * method requires that every chunks implements correctly the
   * {@link Object#equals(Object)} method.
   * 
   * @param message1 a message
   * @param messageManager1Itf the message manager in which the
   *          <code>message1</code> is
   * @param message2 an other message
   * @param messageManager2Itf the message manager in which the
   *          <code>message2</code> is
   * @return <code>true</code> if the two messages has the same content.
   */
  public static boolean equals(final Message message1,
      final MessageManager messageManager1Itf, final Message message2,
      final MessageManager messageManager2Itf)
  {
    if (message1 == message2)
    {
      return true;
    }
    // build a set of chunk's names of message1
    Set<String> names1 = new HashSet<String>();
    Iterator<String> iter = messageManager1Itf.getChunkNameIterator(message1);
    while (iter.hasNext())
    {
      String name = iter.next();
      names1.add(name);
    }
    // check if every chunks in message2 are equals to those in message1
    iter = messageManager2Itf.getChunkNameIterator(message2);
    while (iter.hasNext())
    {
      String chunkName = iter.next();
      names1.remove(chunkName);
      AbstractChunk chunk = message1.getChunk(chunkName);
      if (chunk == null || !chunk.equals(message2.getChunk(chunkName)))
      {
        return false;
      }
    }
    // if names1 is not empty this means that message1 contains chunk's names
    // which are not in message2
    if (!names1.isEmpty())
    {
      return false;
    }

    // check sub messages
    Iterator<Message> iterator1 = messageManager1Itf
        .getSubMessageIterator(message1);
    Iterator<Message> iterator2 = messageManager2Itf
        .getSubMessageIterator(message2);
    while (iterator1.hasNext())
    {
      if (!iterator2.hasNext()
          || (!equals(iterator1.next(), messageManager1Itf, iterator2.next(),
              messageManager2Itf)))
      {
        return false;
      }
    }
    // If iterator2 has next, this means that message2 has more sub messages
    // than message1
    if (iterator2.hasNext())
    {
      return false;
    }
    return true;
  }

  /**
   * Returns an <code>hashCode</code> of the given message based on its
   * content. This {@link Object#hashCode()} method is coherent with the
   * {@link #equals equals} method. This means that if
   * <code>equals(m1, m2) == true</code> then
   * <code>hashCode(m1) == hashCode(m2)</code>.
   * 
   * @param message a message.
   * @param messageManagerItf the message manager in which the message is
   * @return an <code>hashCode</code> of the given message.
   */
  public static int hashCode(final Message message,
      final MessageManager messageManagerItf)
  {
    int hash = 0;
    Iterator<String> iterator = messageManagerItf.getChunkNameIterator(message);
    while (iterator.hasNext())
    {
      String chunkName = iterator.next();
      AbstractChunk chunk = message.getChunk(chunkName);
      hash = append(hash, chunkName);
      hash = append(hash, chunk);
    }

    Iterator<Message> subMsgIterator = messageManagerItf
        .getSubMessageIterator(message);
    while (subMsgIterator.hasNext())
    {
      Message subMessage = subMsgIterator.next();
      hash = append(hash, hashCode(subMessage, messageManagerItf));
    }

    return hash;
  }

  private static int append(int hash, Object o)
  {
    return append(hash, o.hashCode());
  }

  private static int append(int hash, int h)
  {
    return hash * 37 + h;
  }
}
